# (c) 2021 MPIB <https://www.mpib-berlin.mpg.de/>,
#
# This file is part of castellum-scheduler.
#
# castellum-scheduler is free software; you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Castellum is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Castellum. If not, see
# <http://www.gnu.org/licenses/>.

import json

from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.http import HttpResponse
from django.http import StreamingHttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from . import helpers


@method_decorator(csrf_exempt, 'dispatch')
class EventsView(View):
    def dispatch(self, request, *args, **kwargs):
        if request.headers.get('Authorization') != 'token ' + settings.API_TOKEN:
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def stream(self):
        for id, data in helpers.listen():
            yield 'id: {}\ndata: {}\n\n'.format(id, json.dumps(data))

    def get(self, request, *args, **kwargs):
        return StreamingHttpResponse(self.stream(), content_type='text/event-stream')

    def post(self, request, *args, **kwargs):
        try:
            helpers.response(request.GET['id'], 'success' in request.GET)
        except KeyError:
            raise Http404
        return HttpResponse(status=204)
